@@ -56,8 +56,11 @@ import collections
 import copy
 import re
 
+from dateutil import tz
+
 from googlecloudsdk.calliope import parser_errors
 from googlecloudsdk.core import log
+from googlecloudsdk.core import yaml
 from googlecloudsdk.core.console import console_attr
 from googlecloudsdk.core.console import console_io
 from googlecloudsdk.core.util import files
@@ -615,11 +618,11 @@ class Day(object):
 
 
 class Datetime(object):
-  """A class for parsing a datetime object in UTC timezone."""
+  """A class for parsing a datetime object."""
 
   @staticmethod
   def Parse(s):
-    """Parses a string value into a Datetime object."""
+    """Parses a string value into a Datetime object in local timezone."""
     if not s:
       return None
     try:
@@ -630,6 +633,19 @@ class Datetime(object):
               'Failed to parse date/time: {0}'.format(six.text_type(e)),
               user_input=s))
 
+  @staticmethod
+  def ParseUtcTime(s):
+    """Parses a string representing a time in UTC into a Datetime object."""
+    if not s:
+      return None
+    try:
+      return times.ParseDateTime(s, tzinfo=tz.tzutc())
+    except times.Error as e:
+      raise ArgumentTypeError(
+          _GenerateErrorMessage(
+              'Failed to parse UTC time: {0}'.format(six.text_type(e)),
+              user_input=s))
+
 
 class DayOfWeek(object):
   """A class for parsing a day of the week."""
@@ -788,7 +804,7 @@ class ArgList(ArgType):
   ALT_DELIM_CHAR = '^'
 
   def __init__(self, element_type=None, min_length=0, max_length=None,
-               choices=None):
+               choices=None, custom_delim_char=None):
     """Initialize an ArgList.
 
     Args:
@@ -797,6 +813,7 @@ class ArgList(ArgType):
       max_length: int, The maximum size of the list.
       choices: [element_type], a list of valid possibilities for elements. If
           None, then no constraints are imposed.
+      custom_delim_char: char, A customized delimiter character.
 
     Returns:
       (str)->[str], A function to parse the list of values in the argument.
@@ -823,6 +840,8 @@ class ArgList(ArgType):
     self.min_length = min_length
     self.max_length = max_length
 
+    self.custom_delim_char = custom_delim_char
+
   def __call__(self, arg_value):  # pylint:disable=missing-docstring
 
     if isinstance(arg_value, list):
@@ -831,7 +850,7 @@ class ArgList(ArgType):
       raise ArgumentTypeError('Invalid type [{}] for flag value [{}]'.format(
           type(arg_value).__name__, arg_value))
     else:
-      delim = self.DEFAULT_DELIM_CHAR
+      delim = self.custom_delim_char or self.DEFAULT_DELIM_CHAR
       if (arg_value.startswith(self.ALT_DELIM_CHAR) and
           self.ALT_DELIM_CHAR in arg_value[1:]):
         delim, arg_value = arg_value[1:].split(self.ALT_DELIM_CHAR, 1)
@@ -882,7 +901,8 @@ class ArgList(ArgType):
     """
     del is_custom_metavar  # Unused in GetUsageMsg
 
-    required = ','.join([metavar] * self.min_length)
+    delim_char = self.custom_delim_char or self.DEFAULT_DELIM_CHAR
+    required = delim_char.join([metavar] * self.min_length)
 
     if self.max_length:
       num_optional = self.max_length - self.min_length
@@ -895,22 +915,22 @@ class ArgList(ArgType):
     elif num_optional == 1:
       optional = '[{}]'.format(metavar)
     elif num_optional == 2:
-      optional = '[{0},[{0}]]'.format(metavar)
+      optional = '[{0}{1}[{0}]]'.format(metavar, delim_char)
     else:
-      optional = '[{0},...]'.format(metavar)
+      optional = '[{}{}...]'.format(metavar, delim_char)
 
-    msg = ','.join([x for x in [required, optional] if x])
+    msg = delim_char.join([x for x in [required, optional] if x])
 
     if len(msg) < self._MAX_METAVAR_LENGTH:
       return msg
 
     # With long metavars, only put it in once.
     if self.min_length == 0:
-      return '[{},...]'.format(metavar)
+      return '[{}{}...]'.format(metavar, delim_char)
     if self.min_length == 1:
-      return '{},[...]'.format(metavar)
+      return '{}{}[...]'.format(metavar, delim_char)
     else:
-      return '{},...,[...]'.format(metavar)
+      return '{0}{1}...{1}[...]'.format(metavar, delim_char)
 
 
 class ArgDict(ArgList):
@@ -1409,7 +1429,7 @@ class FileContents(object):
   a dangling file handle open. The argument stored in the argparse Namespace
   is the file's contents.
 
-  Args:
+  Attributes:
     binary: bool, If True, the contents of the file will be returned as bytes.
 
   Returns:
@@ -1440,6 +1460,64 @@ class FileContents(object):
       raise ArgumentTypeError(e)
 
 
+class YAMLFileContents(object):
+  """Creates an argparse type that reads the contents of a YAML or JSON file.
+
+  This is similar to argparse.FileType, but unlike FileType it does not leave
+  a dangling file handle open. The argument stored in the argparse Namespace
+  is the file's contents parsed as a YAML object.
+
+  Attributes:
+    validator: function, Function that will validate the provided input
+    file contents.
+
+  Returns:
+    A function that accepts a filename that should be parsed as a YAML
+    or JSON file.
+  """
+
+  def __init__(self, validator=None):
+    if validator and not callable(validator):
+      raise ArgumentTypeError('Validator must be callable')
+    self.validator = validator
+
+  def _AssertJsonLike(self, yaml_data):
+    if not yaml.dict_like(yaml_data) or yaml.list_like(yaml_data):
+      raise ArgumentTypeError('Invalid YAML/JSON Data [{}]'.format(yaml_data))
+
+  def __call__(self, name):
+    """Load YAML data from file path (name).
+
+    If name is "-", stdin is read until EOF. Otherwise, the named file is read.
+    If self.validator is set, call it on the yaml data once it is loaded.
+
+    Args:
+      name: str, The file path to the file.
+
+    Returns:
+      The contents of the file parsed as a YAML data object.
+
+    Raises:
+      ArgumentTypeError: If the file cannot be read or is not a JSON/YAML like
+        object.
+      ValueError: If file content fails validation.
+    """
+    try:
+      if name == '-':
+        yaml_data = yaml.load(console_io.ReadStdin())
+      else:
+        yaml_data = yaml.load_path(name)
+      self._AssertJsonLike(yaml_data)
+      if self.validator:
+        if not self.validator(yaml_data):
+          raise ValueError('Invalid YAML/JSON content [{}]'.format(yaml_data))
+
+      return yaml_data
+
+    except (yaml.YAMLParseError, yaml.FileLoadError) as e:
+      raise ArgumentTypeError(e)
+
+
 class StoreTrueFalseAction(argparse._StoreTrueAction):  # pylint: disable=protected-access
   """Argparse action that acts as a combination of store_true and store_false.
 
@@ -1461,3 +1539,36 @@ class StoreTrueFalseAction(argparse._StoreTrueAction):  # pylint: disable=protec
 
   def __init__(self, *args, **kwargs):
     super(StoreTrueFalseAction, self).__init__(*args, default=None, **kwargs)
+
+
+def StoreFilePathAndContentsAction(binary=False):
+  """Returns Action that stores both file content and file path.
+
+  Args:
+   binary: boolean, whether or not this is a binary file.
+
+  Returns:
+   An argparse action.
+  """
+
+  class Action(argparse.Action):
+    """Stores both file content and file path.
+
+      Stores file contents under original flag DEST and stores file path under
+      DEST_path.
+    """
+
+    def __init__(self, *args, **kwargs):
+      super(Action, self).__init__(*args, **kwargs)
+
+    def __call__(self, parser, namespace, value, option_string=None):
+      """Stores the contents of the file and the file name in namespace."""
+      try:
+        content = console_io.ReadFromFileOrStdin(value, binary=binary)
+      except files.Error as e:
+        raise ArgumentTypeError(e)
+      setattr(namespace, self.dest, content)
+      new_dest = '{}_path'.format(self.dest)
+      setattr(namespace, new_dest, value)
+
+  return Action
