@@ -21,7 +21,8 @@ from __future__ import unicode_literals
 
 from collections import OrderedDict
 import contextlib
-
+import enum
+import getpass
 import io
 import json
 import os
@@ -31,7 +32,6 @@ import sys
 import textwrap
 import threading
 
-import enum
 from googlecloudsdk.core import exceptions
 from googlecloudsdk.core import log
 from googlecloudsdk.core import properties
@@ -53,6 +53,16 @@ class Error(exceptions.Error):
   pass
 
 
+class RequiredPromptError(Error):
+  """An exception for when a prompt cannot silenced with the --quiet flag."""
+
+  def __init__(self):
+    super(RequiredPromptError, self).__init__(
+        'This prompt could not be answered because you are not in an '
+        'interactive session.  Please re-run the command without the --quiet '
+        'flag to respond to the prompts.')
+
+
 class UnattendedPromptError(Error):
   """An exception for when a prompt cannot be answered."""
 
@@ -181,8 +191,8 @@ def IsInteractive(output=False, error=False, heuristic=False):
     # probably being run from a task scheduler context. HOMEPATH can be '\'
     # when a user has a network mapped home directory.
     # Cygwin has it all! Both Windows and Linux. Checking both is perfect.
-    home = os.getenv('HOME')
-    homepath = os.getenv('HOMEPATH')
+    home = encoding.GetEncodedValue(os.environ, 'HOME')
+    homepath = encoding.GetEncodedValue(os.environ, 'HOMEPATH')
     if not homepath and (not home or home == '/'):
       return False
   return True
@@ -449,7 +459,8 @@ def _PrintOptions(options, write, limit=None):
   """
   limited_options = options if limit is None else options[:limit]
   for i, option in enumerate(limited_options):
-    write(' [{index}] {option}\n'.format(index=i + 1, option=str(option)))
+    write(' [{index}] {option}\n'.format(
+        index=i + 1, option=six.text_type(option)))
 
 
 # This defines the point at which, in a PromptChoice, the options
@@ -464,8 +475,8 @@ def PromptChoice(options, default=None, message=None,
   """Prompt the user to select a choice from a list of items.
 
   Args:
-    options:  [object], A list of objects to print as choices.  Their str()
-      method will be used to display them.
+    options:  [object], A list of objects to print as choices.  Their
+      six.text_type() method will be used to display them.
     default: int, The default index to return if prompting is disabled or if
       they do not enter a choice.
     message: str, An optional message to print before the choices are displayed.
@@ -540,7 +551,7 @@ def PromptChoice(options, default=None, message=None,
 
   while True:
     answer = _GetInput()
-    if answer is None or (answer is '' and default is not None):
+    if answer is None or (not answer and default is not None):
       # Return default if we failed to read from stdin.
       # Return default if the user hit enter and there is a valid default,
       # or raise OperationCancelledError if default is the cancel option.
@@ -1056,3 +1067,20 @@ class UXElementType(enum.Enum):
   def GetDataFields(self):
     """Returns the ordered list of additional fields in the UX Element."""
     return self._data_fields
+
+
+def PromptPassword(prompt,
+                   error_message='Invalid Password',
+                   validation_callable=None,
+                   encoder_callable=None):
+  """Prompt user for password with optional validation."""
+  if properties.VALUES.core.disable_prompts.GetBool():
+    return None
+  pass_wd = getpass.getpass(prompt)
+  encoder = encoder_callable if callable(encoder_callable) else six.ensure_str
+  if callable(validation_callable):
+    while not validation_callable(pass_wd):
+      sys.stderr.write(_DoWrap(error_message) + '\n')
+      pass_wd = getpass.getpass(prompt)
+
+  return encoder(pass_wd)
