@@ -549,16 +549,25 @@ class InstallationState(object):
     root = self.__sdk_root
 
     dirs_to_remove = set()
+    pycache_dirs = set()
     for num, p in enumerate(paths, start=1):
       path = os.path.join(root, p)
       if os.path.isfile(path) or os.path.islink(path):
         os.remove(path)
-        # Clean up the pyc files that correspond to any py files being removed.
+        dir_path = os.path.dirname(os.path.normpath(p))
         if p.endswith('.py'):
+          # Python 2 processes leave behind .pyc files adjacent to the .py file;
+          # clean these up for any .py files being removed.
           pyc_path = path + 'c'
           if os.path.isfile(pyc_path):
             os.remove(pyc_path)
-        dir_path = os.path.dirname(os.path.normpath(p))
+          # Python 3 processes leave behind __pycache__ folders in the .py
+          # file's directory; clean these up as well. Since the .pyc files
+          # within have different suffixes depending on the Python version, and
+          # the version of Python that compiled the file may differ from the
+          # current one running, it's faster to just delete the whole folder
+          # later instead of trying to match the file(s) here.
+          pycache_dirs.add(os.path.join(root, dir_path, '__pycache__'))
         while dir_path:
           dirs_to_remove.add(os.path.join(root, dir_path))
           dir_path = os.path.dirname(dir_path)
@@ -567,6 +576,10 @@ class InstallationState(object):
       if progress_callback:
         progress_callback(num / total_paths)
 
+    for d in pycache_dirs:
+      if os.path.isdir(d) and not os.path.islink(d):
+        file_utils.RmTree(d)
+
     # Remove dirs from the bottom up.  Subdirs will always have a longer path
     # than it's parent.
     for d in sorted(dirs_to_remove, key=len, reverse=True):
@@ -598,6 +611,23 @@ class InstallationState(object):
 
     This does not raise exceptions if compiling a given file fails.
     """
+    # Use two different regex exclusions (passed to compile_dir)
+    # based on the python runtime. We package some python code
+    # that is not valid python2 syntax.
+    if six.PY2:
+      regex_exclusion = re.compile('(httplib2/python3|typing/python3'
+                                   '|platform/bq/third_party/yaml/lib3)')
+    else:
+      # Do not compile anything on python 3.4.x
+      if sys.version_info[1] == 4:
+        regex_exclusion = re.compile('.*')
+      elif sys.version_info[1] == 8:
+        regex_exclusion = re.compile(
+            '(kubernetes/utils/create_from_yaml.py'
+            '|gslib/vendored/boto/boto/iam/connection.py)')
+      else:
+        regex_exclusion = None
+
     # The self.sdk_root pathname could contain unicode chars and py_compile
     # chokes on unicode paths. Using relative paths from self.sdk_root works
     # around the problem.
@@ -615,7 +645,7 @@ class InstallationState(object):
         # implementation and bool(2) is True. Starting in python 3.5 this
         # parameter was changed to a multilevel value, where 1 hides files
         # being processed and 2 suppresses output.
-        compileall.compile_dir(d, rx=re.compile('python3'), quiet=2)
+        compileall.compile_dir(d, rx=regex_exclusion, quiet=2, force=True)
 
 
 class InstallationManifest(object):
