@@ -18,23 +18,30 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import unicode_literals
 
+import collections
+import json
 from googlecloudsdk.api_lib.compute import base_classes
+from googlecloudsdk.api_lib.compute import constants
 from googlecloudsdk.api_lib.compute import image_utils
 from googlecloudsdk.api_lib.compute import instance_template_utils
 from googlecloudsdk.api_lib.compute import instance_utils
 from googlecloudsdk.api_lib.compute import metadata_utils
 from googlecloudsdk.api_lib.compute import utils
+from googlecloudsdk.api_lib.compute.instances.create import utils as create_utils
 from googlecloudsdk.calliope import arg_parsers
 from googlecloudsdk.calliope import base
 from googlecloudsdk.calliope import exceptions
 from googlecloudsdk.command_lib.compute import completers
 from googlecloudsdk.command_lib.compute import flags
 from googlecloudsdk.command_lib.compute.instance_templates import flags as instance_templates_flags
+from googlecloudsdk.command_lib.compute.instance_templates import service_proxy_aux_data
 from googlecloudsdk.command_lib.compute.instances import flags as instances_flags
 from googlecloudsdk.command_lib.compute.sole_tenancy import flags as sole_tenancy_flags
 from googlecloudsdk.command_lib.compute.sole_tenancy import util as sole_tenancy_util
 from googlecloudsdk.command_lib.util.args import labels_util
 
+import six
+
 _INSTANTIATE_FROM_VALUES = [
     'attach-read-only',
     'blank',
@@ -52,14 +59,18 @@ def _CommonArgs(
     support_local_ssd_size=False,
     support_kms=False,
     support_resource_policy=False,
-    support_min_node_cpus=False
+    support_min_node_cpu=False,
+    support_location_hint=False,
+    support_multi_writer=True
 ):
   """Adding arguments applicable for creating instance templates."""
   parser.display_info.AddFormat(instance_templates_flags.DEFAULT_LIST_FORMAT)
   metadata_utils.AddMetadataArgs(parser)
   instances_flags.AddDiskArgs(parser, enable_kms=support_kms)
   instances_flags.AddCreateDiskArgs(parser, enable_kms=support_kms,
-                                    resource_policy=support_resource_policy)
+                                    resource_policy=support_resource_policy,
+                                    support_boot=True,
+                                    support_multi_writer=support_multi_writer)
   if support_local_ssd_size:
     instances_flags.AddLocalSsdArgsWithSize(parser)
   else:
@@ -84,8 +95,11 @@ def _CommonArgs(
 
   sole_tenancy_flags.AddNodeAffinityFlagToParser(parser)
 
-  if support_min_node_cpus:
-    sole_tenancy_flags.AddMinNodeCpusArg(parser)
+  if support_min_node_cpu:
+    instances_flags.AddMinNodeCpuArg(parser)
+
+  if support_location_hint:
+    instances_flags.AddLocationHintArg(parser)
 
   flags.AddRegionFlag(
       parser,
@@ -263,12 +277,116 @@ def BuildConfidentialInstanceConfigMessage(messages, args):
   return confidential_instance_config_message
 
 
+def PackageLabels(labels_cls, labels):
+  # Sorted for test stability
+  return labels_cls(additionalProperties=[
+      labels_cls.AdditionalProperty(key=key, value=value)
+      for key, value in sorted(six.iteritems(labels))])
+
+
+# Function copied from labels_util.
+# Temporary fix for adoption tracking of Managed Envoy.
+# TODO(b/146051298) Remove this fix when structured metadata is available.
+def ParseCreateArgsWithServiceProxy(args, labels_cls, labels_dest='labels'):
+  """Initializes labels based on args and the given class."""
+  labels = getattr(args, labels_dest)
+  if getattr(args, 'service_proxy', False):
+    if labels is None:
+      labels = collections.OrderedDict()
+    labels['gce-service-proxy'] = 'on'
+
+  if labels is None:
+    return None
+  return PackageLabels(labels_cls, labels)
+
+
+def AddScopesForServiceProxy(args):
+
+  if getattr(args, 'service_proxy', False):
+    if args.scopes is None:
+      args.scopes = constants.DEFAULT_SCOPES[:]
+
+    if 'cloud-platform' not in args.scopes and 'https://www.googleapis.com/auth/cloud-platform' not in args.scopes:
+      args.scopes.append('cloud-platform')
+
+
+def AddServiceProxyArgsToMetadata(args):
+  """Inserts the Service Proxy arguments provided by the user to the instance metadata.
+
+  Args:
+      args: argparse.Namespace, An object that contains the values for the
+        arguments specified in the .Args() method.
+  """
+  if getattr(args, 'service_proxy', False):
+
+    service_proxy_config = collections.OrderedDict()
+    proxy_spec = collections.OrderedDict()
+
+    service_proxy_config['api-version'] = '0.2'
+
+    # add --service-proxy flag data to metadata.
+    if 'serving-ports' in args.service_proxy:
+      # convert list of strings to list of integers.
+      serving_ports = list(
+          map(int, args.service_proxy['serving-ports'].split(';')))
+      # find unique ports by converting list of integers to set of integers.
+      unique_serving_ports = set(serving_ports)
+      # convert it back to list of integers.
+      # this is done to make it JSON serializable.
+      serving_ports = list(unique_serving_ports)
+      service_proxy_config['service'] = {
+          'serving-ports': serving_ports,
+      }
+
+    if 'proxy-port' in args.service_proxy:
+      proxy_spec['proxy-port'] = args.service_proxy['proxy-port']
+
+    if 'tracing' in args.service_proxy:
+      proxy_spec['tracing'] = args.service_proxy['tracing']
+
+    if 'access-log' in args.service_proxy:
+      proxy_spec['access-log'] = args.service_proxy['access-log']
+
+    if 'network' in args.service_proxy:
+      proxy_spec['network'] = args.service_proxy['network']
+    else:
+      proxy_spec['network'] = ''
+
+    # add --service-proxy-labels flag data to metadata.
+    if getattr(args, 'service_proxy_labels', False):
+      service_proxy_config['labels'] = args.service_proxy_labels
+
+    args.metadata['enable-osconfig'] = 'true'
+    gce_software_declaration = collections.OrderedDict()
+    service_proxy_agent_recipe = collections.OrderedDict()
+
+    service_proxy_agent_recipe['name'] = 'install-gce-service-proxy-agent'
+    service_proxy_agent_recipe['desired_state'] = 'INSTALLED'
+    service_proxy_agent_recipe['installSteps'] = [{
+        'scriptRun': {
+            'script': service_proxy_aux_data.startup_script
+        }
+    }]
+
+    gce_software_declaration['softwareRecipes'] = [service_proxy_agent_recipe]
+
+    args.metadata['gce-software-declaration'] = json.dumps(
+        gce_software_declaration)
+    args.metadata['enable-guest-attributes'] = 'TRUE'
+
+    if proxy_spec:
+      service_proxy_config['proxy-spec'] = proxy_spec
+
+    args.metadata['gce-service-proxy'] = json.dumps(service_proxy_config)
+
+
 def _RunCreate(compute_api,
                args,
                support_source_instance,
                support_kms=False,
-               support_min_node_cpus=False,
-               support_confidential_compute=False):
+               support_min_node_cpu=False,
+               support_confidential_compute=False,
+               support_location_hint=False):
   """Common routine for creating instance template.
 
   This is shared between various release tracks.
@@ -279,10 +397,11 @@ def _RunCreate(compute_api,
         arguments specified in the .Args() method.
       support_source_instance: indicates whether source instance is supported.
       support_kms: Indicate whether KMS is integrated or not.
-      support_min_node_cpus: Indicate whether the --min-node-cpus flag for
+      support_min_node_cpu: Indicate whether the --min-node-cpu flag for
         sole tenancy overcommit is supported.
       support_confidential_compute: Indicate whether confidential compute is
         supported.
+      support_location_hint: Indicate whether location hint is supported.
 
   Returns:
       A resource object dispatched by display.Displayer().
@@ -291,6 +410,8 @@ def _RunCreate(compute_api,
       args, support_kms=support_kms)
   instances_flags.ValidateNetworkTierArgs(args)
 
+  instance_templates_flags.ValidateServiceProxyFlags(args)
+
   client = compute_api.client
 
   boot_disk_size_gb = utils.BytesToGb(args.boot_disk_size)
@@ -300,6 +421,9 @@ def _RunCreate(compute_api,
       Create.InstanceTemplateArg.ResolveAsResource(
           args, compute_api.resources))
 
+  AddScopesForServiceProxy(args)
+  AddServiceProxyArgsToMetadata(args)
+
   metadata = metadata_utils.ConstructMetadataMessage(
       client.messages,
       metadata=args.metadata,
@@ -342,9 +466,13 @@ def _RunCreate(compute_api,
   node_affinities = sole_tenancy_util.GetSchedulingNodeAffinityListFromArgs(
       args, client.messages)
 
-  min_node_cpus = None
-  if support_min_node_cpus and args.IsSpecified('min_node_cpus'):
-    min_node_cpus = args.min_node_cpus
+  min_node_cpu = None
+  if support_min_node_cpu and args.IsSpecified('min_node_cpu'):
+    min_node_cpu = args.min_node_cpu
+
+  location_hint = None
+  if support_location_hint and args.IsSpecified('location_hint'):
+    location_hint = args.location_hint
 
   scheduling = instance_utils.CreateSchedulingMessage(
       messages=client.messages,
@@ -352,7 +480,8 @@ def _RunCreate(compute_api,
       preemptible=args.preemptible,
       restart_on_failure=args.restart_on_failure,
       node_affinities=node_affinities,
-      min_node_cpus=min_node_cpus)
+      min_node_cpu=min_node_cpu,
+      location_hint=location_hint)
 
   if args.no_service_account:
     service_account = None
@@ -363,7 +492,9 @@ def _RunCreate(compute_api,
       scopes=[] if args.no_scopes else args.scopes,
       service_account=service_account)
 
-  create_boot_disk = not instance_utils.UseExistingBootDisk(args.disk or [])
+  create_boot_disk = not (
+      instance_utils.UseExistingBootDisk((args.disk or []) +
+                                         (args.create_disk or [])))
   if create_boot_disk:
     image_expander = image_utils.ImageExpander(client, compute_api.resources)
     try:
@@ -417,13 +548,13 @@ def _RunCreate(compute_api,
   else:
     boot_disk_list = []
 
-  local_nvdimms = instance_utils.CreateLocalNvdimmMessages(
+  local_nvdimms = create_utils.CreateLocalNvdimmMessages(
       args,
       compute_api.resources,
       client.messages,
   )
 
-  local_ssds = instance_utils.CreateLocalSsdMessages(
+  local_ssds = create_utils.CreateLocalSsdMessages(
       args,
       compute_api.resources,
       client.messages,
@@ -471,11 +602,17 @@ def _RunCreate(compute_api,
     instance_template.properties.confidentialInstanceConfig = (
         confidential_instance_config_message)
 
+  if args.private_ipv6_google_access_type is not None:
+    instance_template.properties.privateIpv6GoogleAccess = (
+        instances_flags.GetPrivateIpv6GoogleAccessTypeFlagMapperForTemplate(
+            client.messages).GetEnumForChoice(
+                args.private_ipv6_google_access_type))
+
   request = client.messages.ComputeInstanceTemplatesInsertRequest(
       instanceTemplate=instance_template,
       project=instance_template_ref.project)
 
-  request.instanceTemplate.properties.labels = labels_util.ParseCreateArgs(
+  request.instanceTemplate.properties.labels = ParseCreateArgsWithServiceProxy(
       args, client.messages.InstanceProperties.LabelsValue)
 
   _AddSourceInstanceToTemplate(
@@ -489,7 +626,7 @@ def _RunCreate(compute_api,
 class Create(base.CreateCommand):
   """Create a Compute Engine virtual machine instance template.
 
-  *{command}* facilitates the creation of Google Compute Engine
+  *{command}* facilitates the creation of Compute Engine
   virtual machine instance templates. For example, running:
 
       $ {command} INSTANCE-TEMPLATE
@@ -501,6 +638,8 @@ class Create(base.CreateCommand):
   """
   _support_source_instance = True
   _support_kms = True
+  _support_min_node_cpu = False
+  _support_location_hint = False
 
   @classmethod
   def Args(cls, parser):
@@ -508,9 +647,14 @@ class Create(base.CreateCommand):
         parser,
         release_track=base.ReleaseTrack.GA,
         support_source_instance=cls._support_source_instance,
-        support_kms=cls._support_kms
+        support_kms=cls._support_kms,
+        support_min_node_cpu=cls._support_min_node_cpu,
+        support_location_hint=cls._support_location_hint,
+        support_multi_writer=False
     )
     instances_flags.AddMinCpuPlatformArgs(parser, base.ReleaseTrack.GA)
+    instances_flags.AddPrivateIpv6GoogleAccessArgForTemplate(
+        parser, utils.COMPUTE_GA_API_VERSION)
 
   def Run(self, args):
     """Creates and runs an InstanceTemplates.Insert request.
@@ -526,15 +670,16 @@ class Create(base.CreateCommand):
         base_classes.ComputeApiHolder(base.ReleaseTrack.GA),
         args,
         support_source_instance=self._support_source_instance,
-        support_kms=self._support_kms
-    )
+        support_kms=self._support_kms,
+        support_min_node_cpu=self._support_min_node_cpu,
+        support_location_hint=self._support_location_hint)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.BETA)
 class CreateBeta(Create):
   """Create a Compute Engine virtual machine instance template.
 
-  *{command}* facilitates the creation of Google Compute Engine
+  *{command}* facilitates the creation of Compute Engine
   virtual machine instance templates. For example, running:
 
       $ {command} INSTANCE-TEMPLATE
@@ -547,6 +692,9 @@ class CreateBeta(Create):
   _support_source_instance = True
   _support_kms = True
   _support_resource_policy = True
+  _support_min_node_cpu = True
+  _support_location_hint = False
+  _support_confidential_compute = True
 
   @classmethod
   def Args(cls, parser):
@@ -556,9 +704,15 @@ class CreateBeta(Create):
         support_local_ssd_size=False,
         support_source_instance=cls._support_source_instance,
         support_kms=cls._support_kms,
-        support_resource_policy=cls._support_resource_policy
+        support_resource_policy=cls._support_resource_policy,
+        support_min_node_cpu=cls._support_min_node_cpu,
+        support_location_hint=cls._support_location_hint
     )
     instances_flags.AddMinCpuPlatformArgs(parser, base.ReleaseTrack.BETA)
+    instances_flags.AddPrivateIpv6GoogleAccessArgForTemplate(
+        parser, utils.COMPUTE_BETA_API_VERSION)
+    instances_flags.AddConfidentialComputeArgs(parser)
+    instance_templates_flags.AddServiceProxyConfigArgs(parser)
 
   def Run(self, args):
     """Creates and runs an InstanceTemplates.Insert request.
@@ -574,14 +728,17 @@ class CreateBeta(Create):
         base_classes.ComputeApiHolder(base.ReleaseTrack.BETA),
         args=args,
         support_source_instance=self._support_source_instance,
-        support_kms=self._support_kms)
+        support_kms=self._support_kms,
+        support_min_node_cpu=self._support_min_node_cpu,
+        support_location_hint=self._support_location_hint,
+        support_confidential_compute=self._support_confidential_compute)
 
 
 @base.ReleaseTracks(base.ReleaseTrack.ALPHA)
 class CreateAlpha(Create):
   """Create a Compute Engine virtual machine instance template.
 
-  *{command}* facilitates the creation of Google Compute Engine
+  *{command}* facilitates the creation of Compute Engine
   virtual machine instance templates. For example, running:
 
       $ {command} INSTANCE-TEMPLATE
@@ -594,8 +751,9 @@ class CreateAlpha(Create):
   _support_source_instance = True
   _support_kms = True
   _support_resource_policy = True
-  _support_min_node_cpus = True
+  _support_min_node_cpu = True
   _support_confidential_compute = True
+  _support_location_hint = True
 
   @classmethod
   def Args(cls, parser):
@@ -606,10 +764,15 @@ class CreateAlpha(Create):
         support_source_instance=cls._support_source_instance,
         support_kms=cls._support_kms,
         support_resource_policy=cls._support_resource_policy,
-        support_min_node_cpus=cls._support_min_node_cpus)
+        support_min_node_cpu=cls._support_min_node_cpu,
+        support_location_hint=cls._support_location_hint
+    )
     instances_flags.AddLocalNvdimmArgs(parser)
     instances_flags.AddMinCpuPlatformArgs(parser, base.ReleaseTrack.ALPHA)
     instances_flags.AddConfidentialComputeArgs(parser)
+    instance_templates_flags.AddServiceProxyConfigArgs(parser)
+    instances_flags.AddPrivateIpv6GoogleAccessArgForTemplate(
+        parser, utils.COMPUTE_ALPHA_API_VERSION)
 
   def Run(self, args):
     """Creates and runs an InstanceTemplates.Insert request.
@@ -626,16 +789,17 @@ class CreateAlpha(Create):
         args=args,
         support_source_instance=self._support_source_instance,
         support_kms=self._support_kms,
-        support_min_node_cpus=self._support_min_node_cpus,
-        support_confidential_compute=self._support_confidential_compute)
+        support_min_node_cpu=self._support_min_node_cpu,
+        support_confidential_compute=self._support_confidential_compute,
+        support_location_hint=self._support_location_hint)
 
 
 DETAILED_HELP = {
     'brief':
-        'Create a Compute Engine virtual machine instance template',
+        'Create a Compute Engine virtual machine instance template.',
     'DESCRIPTION':
-        '*{command}* facilitates the creation of Google Compute '
-        'Engine virtual machine instance templates. Instance '
+        '*{command}* facilitates the creation of Compute Engine '
+        'virtual machine instance templates. Instance '
         'templates are global resources, and can be used to create '
         'instances in any zone.',
     'EXAMPLES':
