@@ -1,14 +1,21 @@
 /* Copyright (c) 2009, 2017, Oracle and/or its affiliates. All rights reserved.
  
  This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; version 2 of the License.
- 
+ it under the terms of the GNU General Public License, version 2.0,
+ as published by the Free Software Foundation.
+
+ This program is also distributed with certain software (including
+ but not limited to OpenSSL) that is licensed under separate terms,
+ as designated in a particular file or component or in included license
+ documentation.  The authors of MySQL hereby grant you an additional
+ permission to link the program and your derivative works with the
+ separately licensed software that they have included with MySQL.
+
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
+ GNU General Public License, version 2.0, for more details.
+
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
@@ -322,7 +329,7 @@
 #define HAVE_SIGNAL_H 1
 /* #undef HAVE_SYS_DEVPOLL_H */
 /* #undef HAVE_SYS_EPOLL_H */
-/* #undef HAVE_SYS_EVENT_H */
+#define HAVE_SYS_EVENT_H 1
 #define HAVE_SYS_QUEUE_H 1
 #define HAVE_TAILQFOREACH 1
 #define USE_MB 1
@@ -490,7 +497,7 @@
 
 #define STACK_DIRECTION -1
 
-#define SYSTEM_TYPE "FreeBSD11.2"
+#define SYSTEM_TYPE "FreeBSD12.1"
 #define MACHINE_TYPE "amd64"
 /* #undef HAVE_DTRACE */
 
@@ -639,16 +646,16 @@
 
 #define MYSQL_VERSION_MAJOR 5
 #define MYSQL_VERSION_MINOR 6
-#define MYSQL_VERSION_PATCH 45
+#define MYSQL_VERSION_PATCH 49
 #define MYSQL_VERSION_EXTRA ""
 
 #define PACKAGE "mysql"
 #define PACKAGE_BUGREPORT ""
 #define PACKAGE_NAME "MySQL Server"
-#define PACKAGE_STRING "MySQL Server 5.6.45"
+#define PACKAGE_STRING "MySQL Server 5.6.49"
 #define PACKAGE_TARNAME "mysql"
-#define PACKAGE_VERSION "5.6.45"
-#define VERSION "5.6.45"
+#define PACKAGE_VERSION "5.6.49"
+#define VERSION "5.6.49"
 #define PROTOCOL_VERSION 10
 
 
