@@ -29,6 +29,7 @@
 namespace OPNsense\Auth;
 
 use OPNsense\Core\Config;
+use OPNsense\Core\Backend;
 
 /**
  * Class LDAP connector
@@ -92,9 +93,9 @@ class LDAP extends Base implements IAuthConnector
     private $ldapScope = 'subtree';
 
     /**
-     * @var null|string certificate reference (in /var/run/certs/)
+     * @var null|string url type (standard, startTLS, SSL)
      */
-    private $ldapCAcert = null;
+    private $ldapURLType = null;
 
     /**
      * @var array list of already known usernames vs distinguished names
@@ -111,6 +112,16 @@ class LDAP extends Base implements IAuthConnector
      */
     private $ldapReadProperties = false;
 
+    /**
+     * when set, synchronize groups defined in memberOf attribute to local database
+     */
+    private $ldapSyncMemberOf = false;
+
+    /**
+     * limit the groups which will be considered for sync, empty means all
+     */
+    private $ldapSyncMemberOfLimit = null;
+
     /**
      * @var array internal list of authentication properties (returned by radius auth)
      */
@@ -217,8 +228,11 @@ class LDAP extends Base implements IAuthConnector
         $this->ldapVersion = $ldapVersion;
         $this->baseSearchDN = $baseSearchDN;
         // setup ldap general search list, list gets updated by requested data
-        $this->addSearchAttribute("dn");
+        // Note that the "dn" is always returned irrespective of which attributes types are requested
+        $this->addSearchAttribute("displayName");
+        $this->addSearchAttribute("cn");
         $this->addSearchAttribute("name");
+        $this->addSearchAttribute("mail");
     }
 
     /**
@@ -236,7 +250,9 @@ class LDAP extends Base implements IAuthConnector
             "ldap_authcn" => "ldapAuthcontainers",
             "ldap_scope" => "ldapScope",
             "local_users" => "userDNmap",
-            "ldap_read_properties" => "ldapReadProperties"
+            "ldap_read_properties" => "ldapReadProperties",
+            "ldap_sync_memberof" => "ldapSyncMemberOf",
+            "ldap_sync_memberof_groups" => "ldapSyncMemberOfLimit"
         );
 
         // map properties 1-on-1
@@ -250,46 +266,20 @@ class LDAP extends Base implements IAuthConnector
         // Encryption types: Standard ( none ), StartTLS and SSL
         if (strstr($config['ldap_urltype'], "Standard")) {
             $this->ldapBindURL = "ldap://";
+            $this->ldapURLType = "standard";
         } elseif (strstr($config['ldap_urltype'], "StartTLS")) {
             $this->ldapBindURL = "ldap://";
             $this->useStartTLS = true;
+            $this->ldapURLType = "StartTLS";
         } else {
             $this->ldapBindURL = "ldaps://";
+            $this->ldapURLType = "SSL";
         }
 
         $this->ldapBindURL .= strpos($config['host'], "::") !== false ? "[{$config['host']}]" : $config['host'];
         if (!empty($config['ldap_port'])) {
             $this->ldapBindURL .= ":{$config['ldap_port']}";
         }
-
-        // setup environment
-        if (!empty($config['ldap_caref']) && stristr($config['ldap_urltype'], "standard") === false) {
-            $this->setupCaEnv($config['ldap_caref']);
-        }
-    }
-
-    /**
-     * setup certificate environment
-     * @param string $caref ca reference
-     */
-    public function setupCaEnv($caref)
-    {
-        $this->ldapCAcert = null;
-        if (isset(Config::getInstance()->object()->ca)) {
-            foreach (Config::getInstance()->object()->ca as $cert) {
-                if (isset($cert->refid) && (string)$caref == $cert->refid) {
-                    $this->ldapCAcert = (string)$cert->refid;
-                    @mkdir("/var/run/certs");
-                    @unlink("/var/run/certs/{$this->ldapCAcert}.ca");
-                    file_put_contents("/var/run/certs/{$this->ldapCAcert}.ca", base64_decode((string)$cert->crt));
-                    @chmod("/var/run/certs/{$this->ldapCAcert}.ca", 0644);
-                    break;
-                }
-            }
-        }
-        if (empty($this->ldapCAcert)) {
-            syslog(LOG_ERR, sprintf('LDAP: Could not lookup CA by reference for host %s.', $caref));
-        }
     }
 
     /**
@@ -320,10 +310,9 @@ class LDAP extends Base implements IAuthConnector
         $this->closeLDAPHandle();
 
         // Note: All TLS options must be set before ldap_connect is called
-        if (!empty($this->ldapCAcert)) {
+        if ($this->ldapURLType != "standard") {
             ldap_set_option(null, LDAP_OPT_X_TLS_REQUIRE_CERT, LDAP_OPT_X_TLS_HARD);
-            ldap_set_option(null, LDAP_OPT_X_TLS_CACERTDIR, '/var/run/certs');
-            ldap_set_option(null, LDAP_OPT_X_TLS_CACERTFILE, "/var/run/certs/{$this->ldapCAcert}.ca");
+            ldap_set_option(null, LDAP_OPT_X_TLS_CACERTFILE, "/etc/ssl/cert.pem");
         } else {
             ldap_set_option(null, LDAP_OPT_X_TLS_REQUIRE_CERT, LDAP_OPT_X_TLS_NEVER);
         }
@@ -383,12 +372,25 @@ class LDAP extends Base implements IAuthConnector
                     // fetch distinguished name and most likely username (try the search field first)
                     foreach (array($userNameAttribute, "name") as $ldapAttr) {
                         if (isset($searchResults[$i][$ldapAttr]) && $searchResults[$i][$ldapAttr]['count'] > 0) {
-                            $result[] = array(
+                            $user = array(
                                 'name' => $searchResults[$i][$ldapAttr][0],
-                                'fullname' => !empty($searchResults[$i]['name'][0]) ?
-                                    $searchResults[$i]['name'][0] : "",
                                 'dn' => $searchResults[$i]['dn']
                             );
+                            if (!empty($searchResults[$i]['displayName'][0])) {
+                                $user['fullname'] = $searchResults[$i]['displayName'][0];
+                            } elseif (!empty($searchResults[$i]['cn'][0])) {
+                                $user['fullname'] = $searchResults[$i]['cn'][0];
+                            } elseif (!empty($searchResults[$i]['name'][0])) {
+                                $user['fullname'] = $searchResults[$i]['name'][0];
+                            } else {
+                                $user['fullname'] = '';
+                            }
+                            if (!empty($searchResults[$i]['mail'][0])) {
+                                $user['email'] = $searchResults[$i]['mail'][0];
+                            } else {
+                                $user['email'] = '';
+                            }
+                            $result[] = $user;
                             break;
                         }
                     }
@@ -429,6 +431,91 @@ class LDAP extends Base implements IAuthConnector
         return $this->lastAuthProperties;
     }
 
+    /**
+     * update user group policies when configured
+     * @param string $username authenticated username
+     */
+    private function updatePolicies($username)
+    {
+        if ($this->ldapSyncMemberOf && !empty($this->lastAuthProperties['memberof'])) {
+            $user = $this->getUser($username);
+            // gather known and user configured groups to be able to compare the results from ldap
+            $user_groups = array();
+            $known_groups = array();
+            $cnf = Config::getInstance()->object();
+            if (isset($cnf->system->group)) {
+                foreach ($cnf->system->group as $group) {
+                    $known_groups[] = strtolower((string)$group->name);
+                    // when user is known, collect current groups
+                    if ($user != null && in_array((string)$user->uid, (array)$group->member)) {
+                        $user_groups[] = strtolower((string)$group->name);
+                    }
+                }
+            }
+            // collect all groups from the memberof attribute, store full object path for logging
+            // first cn= defines our local groupname
+            $ldap_groups = array();
+            foreach (explode("\n", $this->lastAuthProperties['memberof']) as $member) {
+                if (stripos($member, "cn=") === 0) {
+                    $ldap_groups[strtolower(explode(",", substr($member, 3))[0])] = $member;
+                }
+            }
+            // list of enabled groups (all when empty), so we can ignore some local groups if needed
+            if (!empty($this->ldapSyncMemberOfLimit)) {
+                $sync_groups = explode(",", strtolower($this->ldapSyncMemberOfLimit));
+            } else {
+                $sync_groups = $known_groups;
+            }
+            //
+            // sort groups and intersect with $sync_groups to determine difference.
+            natcasesort($sync_groups);
+            natcasesort($user_groups);
+            natcasesort($ldap_groups);
+            $diff_ugrp = array_intersect($sync_groups, $user_groups);
+            $diff_lgrp = array_intersect($sync_groups, array_keys($ldap_groups));
+            if ($diff_lgrp != $diff_ugrp) {
+                // update when changed
+                if ($user == null) {
+                    // user creation not supported (yet)
+                    // XXX: we might consider this in the future, since legacy code is involved in creating users
+                    return;
+                }
+                // Lock our configuration while updating, remove now unassigned groups and add new ones
+                // if returned by ldap.
+                $cnf = Config::getInstance()->lock(true)->object();
+                foreach ($cnf->system->group as $group) {
+                    $lc_groupname = strtolower((string)$group->name);
+                    if (in_array($lc_groupname, $sync_groups)) {
+                        if (
+                            in_array((string)$user->uid, (array)$group->member)
+                              && empty($ldap_groups[$lc_groupname])
+                        ) {
+                            unset($group->member[array_search((string)$user->uid, (array)$group->member)]);
+                            syslog(LOG_NOTICE, sprintf(
+                                'User: policy change for %s unlink group %s',
+                                $username,
+                                (string)$group->name
+                            ));
+                        } elseif (
+                            !in_array((string)$user->uid, (array)$group->member)
+                              && !empty($ldap_groups[$lc_groupname])
+                        ) {
+                            syslog(LOG_NOTICE, sprintf(
+                                'User: policy change for %s link group %s [%s]',
+                                $username,
+                                (string)$group->name,
+                                $ldap_groups[$lc_groupname]
+                            ));
+                            $group->addChild('member', (string)$user->uid);
+                        }
+                    }
+                }
+                Config::getInstance()->save();
+                (new Backend())->configdpRun("auth user changed", array($username));
+            }
+        }
+    }
+
     /**
      * authenticate user against ldap server
      * @param string $username username to authenticate
@@ -464,7 +551,6 @@ class LDAP extends Base implements IAuthConnector
                 $sr = @ldap_read($this->ldapHandle, $user_dn, '(objectclass=*)');
                 $info = @ldap_get_entries($this->ldapHandle, $sr);
                 if ($info['count'] != 0) {
-                    // $this->lastAuthProperties['info'] = $info[0];
                     foreach ($info[0] as $ldap_key => $ldap_value) {
                         if (!is_numeric($ldap_key) && $ldap_key !== 'count') {
                             if (isset($ldap_value['count'])) {
@@ -475,6 +561,8 @@ class LDAP extends Base implements IAuthConnector
                             }
                         }
                     }
+                    // update group policies when applicable
+                    $this->updatePolicies($username);
                 }
             }
         }
