@@ -28,8 +28,8 @@
 
 namespace OPNsense\Core;
 
-use \Phalcon\DI\FactoryDefault;
-use \Phalcon\Logger\Adapter\Syslog;
+use Phalcon\DI\FactoryDefault;
+use Phalcon\Logger\Adapter\Syslog;
 
 /**
  * Class Config provides access to systems config xml
@@ -182,7 +182,7 @@ class Config extends Singleton
 
         // root node
         if ($node == null) {
-            $this->simplexml = simplexml_load_string('<'.$this->simplexml[0]->getName().'/>');
+            $this->simplexml = simplexml_load_string('<' . $this->simplexml[0]->getName() . '/>');
             $node = $this->simplexml;
             // invalidate object on warnings/errors (prevent save from happening)
             set_error_handler(
@@ -193,7 +193,8 @@ class Config extends Singleton
         }
 
         foreach ($source as $itemKey => $itemValue) {
-            if ((is_bool($itemValue) && $itemValue == false) ||   // skip empty booleans
+            if (
+                (is_bool($itemValue) && $itemValue == false) ||   // skip empty booleans
                 $itemKey === null || trim($itemKey) === ""        // skip empty tag names
             ) {
                 continue;
@@ -288,12 +289,18 @@ class Config extends Singleton
             if (count($backups) > 0) {
                 // load last backup
                 $logger->error(gettext('No valid config.xml found, attempting last known config restore.'));
-                $this->restoreBackup($backups[0]);
-            } else {
-                // in case there are no backups, restore defaults.
-                $logger->error(gettext('No valid config.xml found, attempting to restore factory config.'));
-                $this->restoreBackup('/usr/local/etc/config.xml');
+                foreach ($backups as $backup) {
+                    try {
+                        $this->restoreBackup($backup);
+                        return;
+                    } catch (ConfigException $e) {
+                        $logger->error("failed restoring " . $backup);
+                    }
+                }
             }
+            // in case there are no backups, restore defaults.
+            $logger->error(gettext('No valid config.xml found, attempting to restore factory config.'));
+            $this->restoreBackup('/usr/local/etc/config.xml');
         }
     }
 
@@ -302,6 +309,10 @@ class Config extends Singleton
      */
     public function forceReload()
     {
+        if ($this->config_file_handle !== null) {
+            fclose($this->config_file_handle);
+            $this->config_file_handle = null;
+        }
         $this->init();
     }
 
@@ -395,7 +406,7 @@ class Config extends Singleton
      * @param array|null $revision revision tag (associative array)
      * @param \SimpleXMLElement|null pass trough xml node
      */
-    private function updateRevision($revision, $node = null)
+    private function updateRevision($revision, $node = null, $timestamp = null)
     {
         // if revision info is not provided, create a default.
         if (!is_array($revision)) {
@@ -407,7 +418,7 @@ class Config extends Singleton
                 $revision['username'] = "(system)";
             }
             if (!empty($_SERVER['REMOTE_ADDR'])) {
-                $revision['username'] .= "@".$_SERVER['REMOTE_ADDR'];
+                $revision['username'] .= "@" . $_SERVER['REMOTE_ADDR'];
             }
             if (!empty($_SERVER['REQUEST_URI'])) {
                 // when update revision is called from a controller, log the endpoint uri
@@ -419,7 +430,7 @@ class Config extends Singleton
         }
 
         // always set timestamp
-        $revision['time'] = microtime(true);
+        $revision['time'] = empty($timestamp) ? microtime(true) : $timestamp;
 
         if ($node == null) {
             if (isset($this->simplexml->revision)) {
@@ -444,22 +455,29 @@ class Config extends Singleton
     }
 
     /**
-     * backup current (running) config
+     * backup current config
+     * @return string target filename
      */
     public function backup()
     {
-        $target_dir = dirname($this->config_file)."/backup/";
-        $target_filename = "config-".microtime(true).".xml";
+        $timestamp = microtime(true);
+        $target_dir = dirname($this->config_file) . "/backup/";
 
         if (!file_exists($target_dir)) {
             // create backup directory if it is missing
             mkdir($target_dir);
         }
-        // The new target backup filename shouldn't exists, because of the use of microtime.
-        // But if for some reason a script keeps calling this backup very often, it should not crash.
-        if (!file_exists($target_dir . $target_filename)) {
-            copy($this->config_file, $target_dir . $target_filename);
+        if (file_exists($target_dir . "config-" . $timestamp . ".xml")) {
+            // The new target backup filename shouldn't exists, because of the use of microtime.
+            // in the unlikely event that we can process events too fast for microtime(), suffix with a more
+            // precise tiestamp to ensure we can't miss a backup
+            $target_filename = "config-" . $timestamp . "_" .  hrtime()[1] . ".xml";
+        } else {
+            $target_filename = "config-" . $timestamp . ".xml";
         }
+        copy($this->config_file, $target_dir . $target_filename);
+
+        return $target_dir . $target_filename;
     }
 
     /**
@@ -470,9 +488,9 @@ class Config extends Singleton
      */
     public function getBackups($fetchRevisionInfo = false)
     {
-        $target_dir = dirname($this->config_file)."/backup/";
+        $target_dir = dirname($this->config_file) . "/backup/";
         if (file_exists($target_dir)) {
-            $backups = glob($target_dir."config*.xml");
+            $backups = glob($target_dir . "config*.xml");
             // sort by date (descending)
             rsort($backups);
             if (!$fetchRevisionInfo) {
@@ -530,17 +548,43 @@ class Config extends Singleton
         }
     }
 
+    /**
+     * @return int number of backups to keep
+     */
+    public function backupCount()
+    {
+        if (
+            $this->statusIsValid && isset($this->simplexml->system->backupcount)
+            && intval($this->simplexml->system->backupcount) >= 0
+        ) {
+            return intval($this->simplexml->system->backupcount);
+        } else {
+            return 100;
+        }
+    }
+
+    /**
+     * return backup file path if revision exists
+     * @param $revision revision timestamp (e.g. 1583766095.9337)
+     * @return bool|string filename when available or false when not found
+     */
+    public function getBackupFilename($revision)
+    {
+        $tmp = preg_replace("/[^0-9.]/", "", $revision);
+        $bckfilename = dirname($this->config_file) . "/backup/config-{$tmp}.xml";
+        if (is_file($bckfilename)) {
+            return $bckfilename;
+        } else {
+            return false;
+        }
+    }
+
     /**
      * remove old backups
      */
     private function cleanupBackups()
     {
-        if ($this->statusIsValid && isset($this->simplexml->system->backupcount)
-                && intval($this->simplexml->system->backupcount) >= 0) {
-            $revisions = intval($this->simplexml->system->backupcount);
-        } else {
-            $revisions = 60;
-        }
+        $revisions = $this->backupCount();
 
         $cnt = 1;
         foreach ($this->getBackups() as $filename) {
@@ -561,23 +605,24 @@ class Config extends Singleton
     {
         $this->checkvalid();
 
-        if ($backup) {
-            $this->backup();
-        }
-
-        // update revision information ROOT.revision tag
-        $this->updateRevision($revision);
-
-        // serialize to text
-        $xml_text = $this->__toString();
+        // update revision information ROOT.revision tag, align timestamp to backup output
+        $this->updateRevision($revision, null, microtime(true));
 
         if ($this->config_file_handle !== null) {
             if (flock($this->config_file_handle, LOCK_EX)) {
                 fseek($this->config_file_handle, 0);
                 ftruncate($this->config_file_handle, 0);
-                fwrite($this->config_file_handle, $xml_text);
+                fwrite($this->config_file_handle, (string)$this);
                 // flush, unlock, but keep the handle open
                 fflush($this->config_file_handle);
+                $backup_filename = $backup ? $this->backup() : null;
+                if ($backup_filename) {
+                    // use syslog to trigger a new configd event, which should signal a syshook config (in batch).
+                    // Althought we include the backup filename, the event handler is responsible to determine the
+                    // last processed event itself. (it's merely added for debug purposes)
+                    $logger = new Syslog("config", array('option' => LOG_PID, 'facility' => LOG_LOCAL5));
+                    $logger->info("config-event: new_config " . $backup_filename);
+                }
                 flock($this->config_file_handle, LOCK_UN);
             } else {
                 throw new ConfigException("Unable to lock config");
