@@ -349,18 +349,20 @@ class Ruleset
             $ownSniffs = $this->expandSniffDirectory($sniffDir, $depth);
         }
 
-        // Included custom autoloaders.
+        // Include custom autoloaders.
         foreach ($ruleset->{'autoload'} as $autoload) {
             if ($this->shouldProcessElement($autoload) === false) {
                 continue;
             }
 
             $autoloadPath = (string) $autoload;
-            if (is_file($autoloadPath) === false) {
-                $autoloadPath = Util\Common::realPath(dirname($rulesetPath).DIRECTORY_SEPARATOR.$autoloadPath);
-            }
 
-            if ($autoloadPath === false) {
+            // Try relative autoload paths first.
+            $relativePath = Util\Common::realPath(dirname($rulesetPath).DIRECTORY_SEPARATOR.$autoloadPath);
+
+            if ($relativePath !== false && is_file($relativePath) === true) {
+                $autoloadPath = $relativePath;
+            } else if (is_file($autoloadPath) === false) {
                 throw new RuntimeException('The specified autoload file "'.$autoload.'" does not exist');
             }
 
@@ -861,11 +863,17 @@ class Ruleset
         $ref  = (string) $rule['ref'];
         $todo = [$ref];
 
-        $parts = explode('.', $ref);
-        if (count($parts) <= 2) {
-            // We are processing a standard or a category of sniffs.
+        $parts      = explode('.', $ref);
+        $partsCount = count($parts);
+        if ($partsCount <= 2 || $partsCount > count(array_filter($parts))) {
+            // We are processing a standard, a category of sniffs or a relative path inclusion.
             foreach ($newSniffs as $sniffFile) {
-                $parts         = explode(DIRECTORY_SEPARATOR, $sniffFile);
+                $parts = explode(DIRECTORY_SEPARATOR, $sniffFile);
+                if (count($parts) === 1 && DIRECTORY_SEPARATOR === '\\') {
+                    // Path using forward slashes while running on Windows.
+                    $parts = explode('/', $sniffFile);
+                }
+
                 $sniffName     = array_pop($parts);
                 $sniffCategory = array_pop($parts);
                 array_pop($parts);
@@ -1183,7 +1191,7 @@ class Ruleset
 
 
     /**
-     * Populates the array of PHP_CodeSniffer_Sniff's for this file.
+     * Populates the array of PHP_CodeSniffer_Sniff objects for this file.
      *
      * @return void
      * @throws \PHP_CodeSniffer\Exceptions\RuntimeException If sniff registration fails.
