@@ -33,7 +33,6 @@ require_once("filter.inc");
 require_once("rrd.inc");
 require_once("system.inc");
 require_once("interfaces.inc");
-require_once("services.inc");
 
 function link_interface_to_vlans($int)
 {
@@ -101,10 +100,20 @@ function list_interfaces() {
                         }
                     }
                 }
+                $interface_item['ifdescr'] = !empty($cnf_item['descr']) ? $cnf_item['descr'] : null;
                 $interfaces[$cnf_item[$value['key']]] = $interface_item;
             }
         }
     }
+    // XXX: get_interface_list() should probably be replaced at some point to avoid traversing through the config
+    //       for all these virtual interfaces
+    $loopbacks = iterator_to_array((new \OPNsense\Interfaces\Loopback())->loopback->iterateItems());
+    foreach ($loopbacks as $loopback) {
+        $interfaces["lo".(string)$loopback->deviceId] = array(
+          'descr' => sprintf("lo%s (%s)", $loopback->deviceId,  $loopback->description),
+          'ifdescr' => sprintf("%s", $loopback->description),
+          'section' => 'loopback');
+    }
 
     // enforce constraints
     foreach ($interfaces as $intf_id => $intf_details) {
@@ -141,10 +150,9 @@ if ($_SERVER['REQUEST_METHOD'] === 'POST') {
         }
 
         $newifname = 'opt' . $i;
-        $descr = 'OPT' . $i;
-
+        $descr = !empty($_POST['new_entry_descr']) ? $_POST['new_entry_descr'] : 'OPT' . $i;
         $config['interfaces'][$newifname] = array();
-        $config['interfaces'][$newifname]['descr'] = $descr;
+        $config['interfaces'][$newifname]['descr'] = preg_replace('/[^a-z_0-9]/i', '', $descr);
         $config['interfaces'][$newifname]['if'] = $_POST['if_add'];
         $interfaces = list_interfaces();
         if ($interfaces[$_POST['if_add']]['section'] == 'ppps.ppp') {
@@ -179,7 +187,11 @@ if ($_SERVER['REQUEST_METHOD'] === 'POST') {
 
             if (isset($config['dhcpd'][$id])) {
                 unset($config['dhcpd'][$id]);
-                services_dhcpd_configure();
+                plugins_configure('dhcp', false, array('inet'));
+            }
+            if (isset($config['dhcpdv6'][$id])) {
+                unset($config['dhcpdv6'][$id]);
+                plugins_configure('dhcp', false, array('inet6'));
             }
             if (isset($config['filter']['rule'])) {
                 foreach ($config['filter']['rule'] as $x => $rule) {
@@ -276,11 +288,16 @@ if ($_SERVER['REQUEST_METHOD'] === 'POST') {
                       $config['interfaces'][$ifname]['ipaddr'] = $interfaces[$ifport]['type'];
                   }
 
-                  if (substr($ifport, 0, 3) == 'gre' || substr($ifport, 0, 3) == 'gif') {
-                      unset($config['interfaces'][$ifname]['ipaddr']);
-                      unset($config['interfaces'][$ifname]['subnet']);
-                      unset($config['interfaces'][$ifname]['ipaddrv6']);
-                      unset($config['interfaces'][$ifname]['subnetv6']);
+                  foreach (plugins_devices() as $device) {
+                      if (!isset($device['configurable']) || $device['configurable'] == true) {
+                          continue;
+                      }
+                      if (preg_match('/' . $device['pattern'] . '/', $ifport)) {
+                          unset($config['interfaces'][$ifname]['ipaddr']);
+                          unset($config['interfaces'][$ifname]['subnet']);
+                          unset($config['interfaces'][$ifname]['ipaddrv6']);
+                          unset($config['interfaces'][$ifname]['subnetv6']);
+                      }
                   }
 
                   /* check for wireless interfaces, set or clear ['wireless'] */
@@ -371,6 +388,14 @@ include("head.inc");
       });
     });
 
+    $("#if_add").change(function(event){
+        event.preventDefault();
+        let descr = $("#if_add option:selected").data('ifdescr');
+        if (descr) {
+            $("#new_entry_descr").val(descr);
+        }
+    });
+
   });
   </script>
 <?php include("fbegin.inc"); ?>
@@ -399,6 +424,7 @@ include("head.inc");
                   <tbody>
 <?php
                   foreach (legacy_config_get_interfaces(array("virtual" => false)) as $ifname => $iface):?>
+                      <?=legacy_html_escape_form_data($iface);?>
                       <tr>
                         <td>
                           <strong><u><span onclick="location.href='/interfaces.php?if=<?=$ifname;?>'" style="cursor: pointer;"><?=$iface['descr'];?></span></u></strong>
@@ -435,12 +461,17 @@ include("head.inc");
 <?php
                           foreach ($unused_interfaces as $portname => $portinfo): ?>
                             <option data-icon="fa fa-plug <?=$portinfo['status'] == 'no carrier' ? "text-danger": "text-success";?>"
+                                    data-ifdescr="<?=!empty($portinfo['ifdescr']) ? $portinfo['ifdescr'] : '';?>"
                                     value="<?=$portname;?>">
                                     <?=$portinfo['descr'];?>
                             </option>
 <?php
                           endforeach; ?>
                           </select>
+                          <div class="form-group">
+                            <label for="new_entry_descr"><?=gettext("Description");?></label>
+                            <input id="new_entry_descr" name="new_entry_descr" type="text" class="form-control">
+                          </form>
                         </td>
                         <td>
                           <button name="add_x" type="submit" value="<?=$portname;?>" class="btn btn-primary" title="<?= html_safe(gettext('Add')) ?>" data-toggle="tooltip">
