@@ -32,7 +32,7 @@ require_once("guiconfig.inc");
 require_once("filter.inc");
 require_once("system.inc");
 require_once("interfaces.inc");
-require_once("services.inc");
+require_once("plugins.inc.d/dhcpd.inc");
 
 function reconfigure_dhcpd()
 {
@@ -64,7 +64,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         $pconfig['prefixrange_length'] = $config['dhcpdv6'][$if]['prefixrange']['prefixlength'];
     }
     $config_copy_fieldsnames = array('defaultleasetime', 'maxleasetime', 'domain', 'domainsearchlist', 'ddnsdomain',
-        'ddnsdomainprimary', 'ddnsdomainkeyname', 'ddnsdomainkey', 'bootfile_url', 'netmask',
+        'ddnsdomainprimary', 'ddnsdomainkeyname', 'ddnsdomainkey', 'ddnsdomainalgorithm', 'bootfile_url', 'netmask',
         'numberoptions', 'dhcpv6leaseinlocaltime', 'staticmap');
     foreach ($config_copy_fieldsnames as $fieldname) {
         if (isset($config['dhcpdv6'][$if][$fieldname])) {
@@ -86,6 +86,11 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     $pconfig['ntp1'] = !empty($config['dhcpdv6'][$if]['ntpserver'][0]) ? $config['dhcpdv6'][$if]['ntpserver'][0] : "";
     $pconfig['ntp2'] = !empty($config['dhcpdv6'][$if]['ntpserver'][1]) ? $config['dhcpdv6'][$if]['ntpserver'][1] : "";
 
+    // backward compatibility: migrate 'domain' to 'domainsearchlist'
+    if (empty($pconfig['domainsearchlist'])) {
+        $pconfig['domainsearchlist'] = $pconfig['domain'];
+    }
+
 } elseif ($_SERVER['REQUEST_METHOD'] === 'POST') {
     // handle identifiers and actions
     if (!empty($_POST['if']) && !empty($config['interfaces'][$_POST['if']])) {
@@ -176,9 +181,6 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         if ((!empty($pconfig['ntp1']) && !is_ipaddrv6($pconfig['ntp1'])) || (!empty($pconfig['ntp2']) && !is_ipaddrv6($pconfig['ntp2']))) {
             $input_errors[] = gettext("A valid IPv6 address must be specified for the primary/secondary NTP servers.");
         }
-        if (!empty($pconfig['domain']) && !is_domain($pconfig['domain'])) {
-            $input_errors[] = gettext("A valid domain name must be specified for the DNS domain.");
-        }
         if (!empty($pconfig['bootfile_url']) && !is_URL($pconfig['bootfile_url'])) {
             $input_errors[] = gettext("A valid URL must be specified for the network bootfile.");
         }
@@ -237,8 +239,8 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             $dhcpdconf = array();
 
             // simple 1-on-1 copy
-            $config_copy_fieldsnames = array('defaultleasetime', 'maxleasetime', 'netmask', 'domain', 'domainsearchlist',
-              'ddnsdomain', 'ddnsdomainprimary', 'ddnsdomainkeyname', 'ddnsdomainkey', 'bootfile_url',
+            $config_copy_fieldsnames = array('defaultleasetime', 'maxleasetime', 'netmask', 'domainsearchlist',
+              'ddnsdomain', 'ddnsdomainprimary', 'ddnsdomainkeyname', 'ddnsdomainkey', 'ddnsdomainalgorithm', 'bootfile_url',
               'dhcpv6leaseinlocaltime');
             foreach ($config_copy_fieldsnames as $fieldname) {
                 if (!empty($pconfig[$fieldname])) {
@@ -285,7 +287,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             }
             // router advertisement data lives in the same spot, copy
             foreach (array_keys($config['dhcpdv6'][$if]) as $fieldname) {
-                if (substr($fieldname,0 ,2) == 'ra' && !in_array($fieldname, array_keys($dhcpdconf))) {
+                if ((substr($fieldname, 0, 2) == 'ra' || substr($fieldname, 0, 3) == 'Adv') && !in_array($fieldname, array_keys($dhcpdconf))) {
                     $dhcpdconf[$fieldname] = $config['dhcpdv6'][$if][$fieldname];
                 }
             }
@@ -316,7 +318,6 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         }
         exit;
     }
-
 }
 
 $service_hook = 'dhcpd6';
@@ -325,7 +326,8 @@ legacy_html_escape_form_data($pconfig);
 
 include("head.inc");
 
-list ($wifcfgip, $wifcfgsn) = explode('/', find_interface_networkv6(get_real_interface($if, 'inet6'), false));
+list ($wifcfgip, $networkv6) = interfaces_primary_address6($if);
+$wifcfgsn = explode('/', $networkv6)[1];
 
 if (isset($config['interfaces'][$if]['dhcpd6track6allowoverride'])) {
     $prefix_array = array();
@@ -556,21 +558,12 @@ if (isset($config['interfaces'][$if]['dhcpd6track6allowoverride'])) {
                         </div>
                       </td>
                     </tr>
-                    <tr>
-                      <td><a id="help_for_domain" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Domain name");?></td>
-                      <td>
-                        <input name="domain" type="text" id="domain" value="<?=$pconfig['domain'];?>" />
-                        <div class="hidden" data-for="help_for_domain">
-                          <?=gettext("The default is to use the domain name of this system as the default domain name provided by DHCP. You may specify an alternate domain name here.");?>
-                        </div>
-                     </td>
-                    </tr>
                     <tr>
                       <td><a id="help_for_domainsearchlist" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Domain search list");?></td>
                       <td>
                         <input name="domainsearchlist" type="text" id="domainsearchlist" value="<?=$pconfig['domainsearchlist'];?>" />
                         <div class="hidden" data-for="help_for_domainsearchlist">
-                          <?=gettext("The DHCP server can optionally provide a domain search list. Use the semicolon character as separator.");?>
+                          <?=gettext("The default is to use the domain name of this system as the domain search list option provided by DHCPv6. You may optionally specify one or multiple domain(s) here. Use the semicolon character as separator. The first domain in this list will also be used for DNS registration of DHCP static mappings (if enabled).");?>
                         </div>
                       </td>
                     </tr>
@@ -624,6 +617,16 @@ if (isset($config['interfaces'][$if]['dhcpd6track6allowoverride'])) {
                           <input name="ddnsdomainkeyname" type="text" id="ddnsdomainkeyname" size="20" value="<?=$pconfig['ddnsdomainkeyname'];?>" />
                           <?=gettext("Enter the dynamic DNS domain key secret which will be used to register client names in the DNS server.");?>
                           <input name="ddnsdomainkey" type="text" id="ddnsdomainkey" size="20" value="<?=$pconfig['ddnsdomainkey'];?>" />
+                          <?=gettext("Choose the dynamic DNS domain key algorithm.");?><br />
+                          <select name='ddnsdomainalgorithm' id="ddnsdomainalgorithm" class="selectpicker">
+<?php
+                          foreach (array("hmac-md5", "hmac-sha512") as $algorithm) :?>
+                              <option value="<?=$algorithm;?>" <?=$pconfig['ddnsdomainalgorithm'] == $algorithm ? "selected=\"selected\"" :"";?>>
+                                <?=$algorithm;?>
+                              </option>
+<?php
+                          endforeach; ?>
+                          </select>
                         </div>
                       </td>
                     </tr>
