@@ -1,7 +1,7 @@
 <?php
 
 /*
- * Copyright (C) 2017-2018 Franco Fichtner <franco@opnsense.org>
+ * Copyright (C) 2017-2019 Franco Fichtner <franco@opnsense.org>
  * Copyright (C) 2014-2015 Deciso B.V.
  * Copyright (C) 2005-2010 Scott Ullrich <sullrich@gmail.com>
  * Copyright (C) 2008 Shrew Soft Inc. <mgrooms@shrew.net>
@@ -34,7 +34,6 @@
 require_once("guiconfig.inc");
 require_once("filter.inc");
 require_once("system.inc");
-require_once("services.inc");
 
 $a_group = &config_read_array('system', 'group');
 $a_authmode = auth_get_authserver_list();
@@ -56,6 +55,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     $pconfig['usevirtualterminal'] = isset($config['system']['usevirtualterminal']);
     $pconfig['disableintegratedauth'] = !empty($config['system']['disableintegratedauth']);
     $pconfig['sudo_allow_wheel'] = $config['system']['sudo_allow_wheel'];
+    $pconfig['sudo_allow_group'] = isset($config['system']['sudo_allow_group']) ? $config['system']['sudo_allow_group'] : null;
     $pconfig['nodnsrebindcheck'] = isset($config['system']['webgui']['nodnsrebindcheck']);
     $pconfig['nohttpreferercheck'] = isset($config['system']['webgui']['nohttpreferercheck']);
     $pconfig['althostnames'] = $config['system']['webgui']['althostnames'];
@@ -66,6 +66,11 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
     $pconfig['enablesshd'] = $config['system']['ssh']['enabled'];
     $pconfig['sshport'] = $config['system']['ssh']['port'];
     $pconfig['sshinterfaces'] = !empty($config['system']['ssh']['interfaces']) ? explode(',', $config['system']['ssh']['interfaces']) : array();
+    $pconfig['ssh-kex'] = !empty($config['system']['ssh']['kex']) ? explode(',', $config['system']['ssh']['kex']) : array();
+    $pconfig['ssh-ciphers'] = !empty($config['system']['ssh']['ciphers']) ? explode(',', $config['system']['ssh']['ciphers']) : array();
+    $pconfig['ssh-macs'] = !empty($config['system']['ssh']['macs']) ? explode(',', $config['system']['ssh']['macs']) : array();
+    $pconfig['ssh-keys'] = !empty($config['system']['ssh']['keys']) ? explode(',', $config['system']['ssh']['keys']) : array();
+
     /* XXX listtag "fun" */
     $pconfig['sshlogingroup'] = !empty($config['system']['ssh']['group'][0]) ? $config['system']['ssh']['group'][0] : null;
     $pconfig['sshpasswordauth'] = isset($config['system']['ssh']['passwordauth']);
@@ -235,6 +240,12 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
 
         $config['system']['ssh']['interfaces'] = !empty($pconfig['sshinterfaces']) ? implode(',', $pconfig['sshinterfaces']) : null;
 
+        $config['system']['ssh']['kex'] = !empty($pconfig['ssh-kex']) ? implode(',', $pconfig['ssh-kex']) : null;
+        $config['system']['ssh']['ciphers'] = !empty($pconfig['ssh-ciphers']) ? implode(',', $pconfig['ssh-ciphers']) : null;
+        $config['system']['ssh']['macs'] = !empty($pconfig['ssh-macs']) ? implode(',', $pconfig['ssh-macs']) : null;
+        $config['system']['ssh']['keys'] = !empty($pconfig['ssh-keys']) ? implode(',', $pconfig['ssh-keys']) : null;
+
+
         if (!empty($pconfig['enablesshd'])) {
             $config['system']['ssh']['enabled'] = 'enabled';
         } elseif (isset($config['system']['ssh']['enabled'])) {
@@ -247,6 +258,12 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
             unset($config['system']['ssh']['group']);
         }
 
+        if (!empty($pconfig['sudo_allow_group'])) {
+            $config['system']['sudo_allow_group'] = $pconfig['sudo_allow_group'];
+        } elseif (isset($config['system']['sudo_allow_group'])) {
+            unset($config['system']['sudo_allow_group']);
+        }
+
         if (!empty($pconfig['sshpasswordauth'])) {
             $config['system']['ssh']['passwordauth'] = true;
         } elseif (isset($config['system']['ssh']['passwordauth'])) {
@@ -297,7 +314,7 @@ if ($_SERVER['REQUEST_METHOD'] === 'GET') {
         system_login_configure();
         system_hosts_generate();
         plugins_configure('dns');
-        services_dhcpd_configure();
+        plugins_configure('dhcp');
         configd_run('openssh restart', true);
 
         if ($restart_webgui) {
@@ -356,7 +373,7 @@ $(document).ready(function() {
                  btnCancelLabel: '<?= html_safe(gettext('Use the default')) ?>',
                  callback: function(result) {
                      if (!result) {
-                         $('#webguiinterface option:selected').removeAttr('selected');
+                         $('#webguiinterface option:selected').prop('selected', false);
                          $('#webguiinterface').selectpicker('refresh');
                          $.webguiinterface_warned = 0;
                      }
@@ -364,7 +381,7 @@ $(document).ready(function() {
              });
          }
      });
-     $.webguiinterface_warned = 0;
+     $.webguiinterface_warned = $('#webguiinterface option:selected').length ? 1 : 0;
 
  <?php
     if (isset($restart_webgui) && $restart_webgui): ?>
@@ -453,13 +470,13 @@ $(document).ready(function() {
                 <td><a id="help_for_sslcertref" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("SSL Certificate"); ?></td>
                 <td>
                   <select name="ssl-certref" class="selectpicker" data-style="btn-default">
-<?php
-                  foreach ($a_cert as $cert) :?>
+<?php foreach ($a_cert as $cert): ?>
+<?php if (isset($cert['prv'])): ?>
                     <option value="<?=$cert['refid'];?>" <?=$pconfig['ssl-certref'] == $cert['refid'] ? "selected=\"selected\"" : "";?>>
                       <?=$cert['descr'];?>
                     </option>
-<?php
-                  endforeach;?>
+<?php endif ?>
+<?php endforeach ?>
                   </select>
                   <div class='hidden' data-for="help_for_sslcertref">
                     <?=sprintf(
@@ -603,13 +620,11 @@ $(document).ready(function() {
               </tr>
               <tr>
                 <td><a id="help_for_webguiinterfaces" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext('Listen Interfaces') ?></td>
-                  <td>
+                <td>
                   <select id="webguiinterface" name="webguiinterfaces[]" multiple="multiple" class="selectpicker" title="<?= html_safe(gettext('All (recommended)')) ?>">
-<?php
-                  foreach ($interfaces as $iface => $ifacename): ?>
+<?php foreach ($interfaces as $iface => $ifacename): ?>
                       <option value="<?= html_safe($iface) ?>" <?= !empty($pconfig['webguiinterfaces']) && in_array($iface, $pconfig['webguiinterfaces']) ? 'selected="selected"' : '' ?>><?= html_safe($ifacename) ?></option>
-<?php
-                  endforeach;?>
+<?php endforeach ?>
                   </select>
                   <div class="hidden" data-for="help_for_webguiinterfaces">
                     <?= gettext('Only accept connections from the selected interfaces. Leave empty to listen globally. Use with care.') ?>
@@ -647,14 +662,12 @@ $(document).ready(function() {
               </tr>
               <tr>
                 <td><a id="help_for_sshlogingroup" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext('Login Group') ?></td>
-                  <td>
+                <td>
                   <select name="sshlogingroup" class="selectpicker">
                       <option value=""><!-- do not translate: -->wheel</option>
-<?php
-                  foreach ($a_group as $group) :?>
+<?php foreach ($a_group as $group): ?>
                       <option value="<?= html_safe($group['name']) ?>" <?= $pconfig['sshlogingroup'] == $group['name'] ? 'selected="selected"' : '' ?>><!-- do not translate: -->wheel, <?= html_safe($group['name']) ?></option>
-<?php
-                  endforeach;?>
+<?php endforeach ?>
                   </select>
                   <div class="hidden" data-for="help_for_sshlogingroup">
                     <?= gettext('Select the allowed groups for remote login. The "wheel" group is always set for recovery purposes and an additional local group can be selected at will. Do not yield remote access to non-adminstrators as every user can access system files using SSH or SFTP.') ?>
@@ -696,19 +709,89 @@ $(document).ready(function() {
               </tr>
               <tr>
                 <td><a id="help_for_sshinterfaces" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext('Listen Interfaces') ?></td>
-                  <td>
+                <td>
                   <select name="sshinterfaces[]" multiple="multiple" class="selectpicker" title="<?= html_safe(gettext('All (recommended)')) ?>">
-<?php
-                  foreach ($interfaces as $iface => $ifacename): ?>
+<?php foreach ($interfaces as $iface => $ifacename): ?>
                       <option value="<?= html_safe($iface) ?>" <?= !empty($pconfig['sshinterfaces']) && in_array($iface, $pconfig['sshinterfaces']) ? 'selected="selected"' : '' ?>><?= html_safe($ifacename) ?></option>
-<?php
-                  endforeach;?>
+<?php endforeach ?>
                   </select>
                   <div class="hidden" data-for="help_for_sshinterfaces">
                     <?= gettext('Only accept connections from the selected interfaces. Leave empty to listen globally. Use with care.') ?>
                   </div>
                 </td>
               </tr>
+              <tr>
+                <td><a id="help_for_sshkex" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Key exchange algorithms"); ?></td>
+                <td>
+                    <select name="ssh-kex[]" class="selectpicker" multiple="multiple" data-live-search="true" title="<?=gettext("System defaults");?>">
+<?php
+                    $options = json_decode(configd_run("openssh query kex"), true);
+                    foreach ($options = empty($options) ? array() : $options as $option):?>
+                      <option value="<?=$option;?>" <?= !empty($pconfig['ssh-kex']) && in_array($option, $pconfig['ssh-kex']) ? 'selected="selected"' : '' ?>>
+                        <?=$option;?>
+                      </option>
+<?php
+                    endforeach;?>
+                    </select>
+                    <div class="hidden" data-for="help_for_sshkex">
+                      <?=gettext("The key exchange methods that are used to generate per-connection keys");?>
+                    </div>
+                </td>
+              </tr>
+              <tr>
+                <td><a id="help_for_sshciphers" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Ciphers"); ?></td>
+                <td>
+                    <select name="ssh-ciphers[]" class="selectpicker" multiple="multiple" data-live-search="true" title="<?=gettext("System defaults");?>">
+<?php
+                    $options = json_decode(configd_run("openssh query cipher"), true);
+                    foreach ($options = empty($options) ? array() : $options as $option):?>
+                      <option value="<?=$option;?>" <?= !empty($pconfig['ssh-ciphers']) && in_array($option, $pconfig['ssh-ciphers']) ? 'selected="selected"' : '' ?>>
+                        <?=$option;?>
+                      </option>
+<?php
+                    endforeach;?>
+                    </select>
+                    <div class="hidden" data-for="help_for_sshciphers">
+                      <?=gettext("The ciphers to encrypt the connection");?>
+                    </div>
+                </td>
+              </tr>
+              <tr>
+                <td><a id="help_for_sshmacs" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("MACs"); ?></td>
+                <td>
+                    <select name="ssh-macs[]" class="selectpicker" multiple="multiple" data-live-search="true" title="<?=gettext("System defaults");?>">
+<?php
+                    $options = json_decode(configd_run("openssh query mac"), true);
+                    foreach ($options = empty($options) ? array() : $options as $option):?>
+                      <option value="<?=$option;?>" <?= !empty($pconfig['ssh-macs']) && in_array($option, $pconfig['ssh-macs']) ? 'selected="selected"' : '' ?>>
+                        <?=$option;?>
+                      </option>
+<?php
+                    endforeach;?>
+                    </select>
+                    <div class="hidden" data-for="help_for_sshmacs">
+                      <?=gettext("The message authentication codes used to detect traffic modification");?>
+                    </div>
+                </td>
+              </tr>
+              <tr>
+                <td><a id="help_for_sshkeys" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?=gettext("Host key algorithms"); ?></td>
+                <td>
+                    <select name="ssh-keys[]" class="selectpicker" multiple="multiple" data-live-search="true" title="<?=gettext("System defaults");?>">
+<?php
+                    $options = json_decode(configd_run("openssh query key"), true);
+                    foreach ($options = empty($options) ? array() : $options as $option):?>
+                      <option value="<?=$option;?>" <?= !empty($pconfig['ssh-keys']) && in_array($option, $pconfig['ssh-keys']) ? 'selected="selected"' : '' ?>>
+                        <?=$option;?>
+                      </option>
+<?php
+                    endforeach;?>
+                    </select>
+                    <div class="hidden" data-for="help_for_sshkeys">
+                      <?=gettext("Specifies the host	key algorithms that the	server offers");?>
+                    </div>
+                </td>
+              </tr>
             </table>
           </div>
           <div class="content-box tab-content table-responsive __mb">
@@ -795,13 +878,11 @@ $(document).ready(function() {
                 <td><a id="help_for_authmode" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?= gettext('Server') ?></td>
                 <td>
                   <select name="authmode[]" multiple="multiple" class="selectpicker" data-style="btn-default">
-<?php
-                  foreach ($a_authmode as $auth_key => $auth_server): ?>
+<?php foreach ($a_authmode as $auth_key => $auth_server): ?>
                     <option value="<?= html_safe($auth_key) ?>" <?= !empty($pconfig['authmode']) && in_array($auth_key, $pconfig['authmode']) ? 'selected="selected"' : '' ?>>
                       <?= html_safe($auth_server['name']) ?>
                     </option>
-<?php
-                  endforeach ?>
+<?php endforeach ?>
                   </select>
                   <div class="hidden" data-for="help_for_authmode">
                     <?= gettext('Select one or more authentication servers to validate user credentials against. ' .
@@ -810,6 +891,16 @@ $(document).ready(function() {
                   </div>
                 </td>
               </tr>
+              <tr>
+                <td></td>
+                <td>
+                  <input name="disableintegratedauth" type="checkbox" value="yes" <?= empty($pconfig['disableintegratedauth']) ? '' : 'checked="checked"' ?>  />
+                  <?=gettext("Disable integrated authentication"); ?>
+                  <div class="hidden" data-for="help_for_authmode">
+                    <?= gettext('When set, console login, SSH, and other system services can only use standard UNIX account authentication.') ?>
+                  </div>
+                </td>
+              </tr>
               <tr>
                 <td><a id="help_for_sudo_allow_wheel" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?= gettext('Sudo') ?></td>
                 <td>
@@ -824,12 +915,16 @@ $(document).ready(function() {
                 </td>
               </tr>
               <tr>
-                <td><a id="help_for_disableintegratedauth" href="#" class="showhelp"><i class="fa fa-info-circle"></i></a> <?= gettext('System') ?></td>
+                <td></td>
                 <td>
-                  <input name="disableintegratedauth" type="checkbox" value="yes" <?= empty($pconfig['disableintegratedauth']) ? '' : 'checked="checked"' ?>  />
-                  <?=gettext("Disable integrated authentication"); ?>
-                  <div class="hidden" data-for="help_for_disableintegratedauth">
-                    <?= gettext('When set, console login, SSH, and other system services can only use standard UNIX account authentication.') ?>
+                  <select name="sudo_allow_group" class="selectpicker">
+                      <option value=""><!-- do not translate: -->wheel</option>
+<?php foreach ($a_group as $group): ?>
+                      <option value="<?= html_safe($group['name']) ?>" <?= $pconfig['sudo_allow_group'] == $group['name'] ? 'selected="selected"' : '' ?>><!-- do not translate: -->wheel, <?= html_safe($group['name']) ?></option>
+<?php endforeach ?>
+                  </select>
+                  <div class="hidden" data-for="help_for_sudo_allow_wheel">
+                    <?= gettext('Select the allowed groups for sudo usage. The "wheel" group is always set for recovery purposes and an additional local group can be selected at will.') ?>
                   </div>
                 </td>
               </tr>
